/******************************************************************
	This is a IO port driver.
	
******************************************************************/




#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/slab.h>
#include <linux/clk.h>
#include <linux/err.h>
#include <linux/io.h>
#include <linux/ioport.h>
#include <linux/pci.h>
#include <linux/init.h>
#include <linux/mutex.h>
#include <linux/sysfs.h>
#include <linux/delay.h>
#include <linux/input.h>



#define DBG	0
#define DEVICE_NAME "tp8m"
#define CONTROL_DRIVER_VERSION "DATE 07-25-2018 VER: 0.01"
#define LOBYTE(x) ((unsigned char)((x) & 0xff))
#define HIBYTE(x) ((unsigned char)(((x) >> 8) & 0xff))
#define LOWORD(x) ((unsigned short int)((x) & 0xffff))
#define HIWORD(x) ((unsigned short int)(((x) >> 16) & 0xffff))

#define dim(x) ( sizeof(x)/sizeof(x[0]) )

#define DEF_VAL_ON	1
#define DEF_VAL_OFF	0

#define DIR_OUTPUT	1
#define DIR_INPUT	0

#define u8 unsigned char
#define u16 unsigned int
#define PSTR unsigned char*


#define IBF_BIT 1<<1
#define OBF_BIT 1<<0

#define EC_PORT_INDEX 0x66
#define EC_PORT_DATA 0x62

#define TIMEOUT 160
#define DELAY_US 4000
u16 count;
#define LOOP_OBF_NOTEQ_TRUE \
	count=TIMEOUT;\
	while(count--) {\
		udelay(DELAY_US);\
		if( (inb(EC_PORT_INDEX)& OBF_BIT)!=OBF_BIT )\
			break;\
	} 

#define LOOP_IBF_NOTEQ_TRUE \
	count=TIMEOUT;\
	while(count--) {\
		udelay(DELAY_US);\
		if( (inb(EC_PORT_INDEX)& IBF_BIT)!=IBF_BIT )\
			 break;\
	}

#define LOOP_OBF_EQ_TRUE \
	count=TIMEOUT;\
	while(count--) {\
		udelay(DELAY_US);\
		if( (inb(EC_PORT_INDEX)& OBF_BIT)==OBF_BIT )\
			 break;\
	} 
	

static struct platform_device *pdev;


struct GPIO_DEF{
  unsigned char* name;
  u16 port;
  u8 bit;
  u8 def_val;
  u8 dir;
};



#define SETTING_GPO1			"GPO1"
#define SETTING_GPO2			"GPO2"


struct GPIO_DEF gpio_table[]=
{
 
  {	SETTING_GPO2,						0x221,	1<<3,	DEF_VAL_OFF,	DIR_OUTPUT	},
  {	SETTING_GPO1,						0x221,	1<<4,	DEF_VAL_OFF,	DIR_OUTPUT	},
};




/*--------------------------------------------------------------------------------------------------------------------------------------------
						      BASE IO FUNCTION
 ---------------------------------------------------------------------------------------------------------------------------------------------*/
ssize_t read_IO_value(unsigned char* name, char *buf)
{
  u8 i, bit,gpioin;

  
  for(i=0; i<dim(gpio_table); i++)
  {
    if(strcmp(name,gpio_table[i].name)==0)
    {
					LOOP_IBF_NOTEQ_TRUE
					outb(0xE4, 0x66);
					LOOP_OBF_EQ_TRUE
					gpioin = inb(0x62);	
					
      bit=gpioin;
      bit=bit&gpio_table[i].bit;
      return sprintf(buf, "%x\n", bit? 1:0);
    }
  }
  return -1;
}

ssize_t write_IO_value(unsigned char* name, const char *buf)
{
  u8 i, value, action_value;
  
  action_value = simple_strtoul(buf, NULL, 0);
  for(i=0; i<dim(gpio_table); i++)
  {
    if(strcmp(name,gpio_table[i].name)==0)
    {
					LOOP_IBF_NOTEQ_TRUE
					outb(0xE4, 0x66);
					LOOP_OBF_EQ_TRUE
					value = inb(0x62);
					if(i==0)
					{
						action_value = value & 0xFD;
						LOOP_IBF_NOTEQ_TRUE
						outb(0xE1, 0x66);
						LOOP_IBF_NOTEQ_TRUE
						outb(action_value, 0x62);
					}
					else if(i==1)
					{
						action_value = value & 0xFB;
						LOOP_IBF_NOTEQ_TRUE
						outb(0xE1, 0x66);
						LOOP_IBF_NOTEQ_TRUE
						outb(action_value, 0x62);

					}
			udelay(1000);
					action_value = value | 0x06;
					
					LOOP_IBF_NOTEQ_TRUE
					outb(0xE1, 0x66);
					LOOP_IBF_NOTEQ_TRUE
					outb(action_value, 0x62);
					
					printk("Cash Drawer Write complete\n");
					
     return 0;
    }
  }
 
  return -1;
}

/*-----------------------------------------------------------------------------------------------------------------------------------------------------------
									      SYSFS FUNCTION
 ------------------------------------------------------------------------------------------------------------------------------------------------------------*/

//+
static ssize_t tp8m_GPO_GPO2_read(struct device *dev,struct device_attribute *attr, char *buf)
{
  return read_IO_value(SETTING_GPO2, buf);
}

static ssize_t tp8m_GPO_GPO2_write(struct device *dev, struct device_attribute *attr, const char *buf, size_t len)
{
 write_IO_value(SETTING_GPO2, buf);
 return len;
}
//-


//+
static ssize_t tp8m_GPO_GPO1_read(struct device *dev,struct device_attribute *attr, char *buf)
{
  return read_IO_value(SETTING_GPO1, buf);
}

static ssize_t tp8m_GPO_GPO1_write(struct device *dev, struct device_attribute *attr, const char *buf, size_t len)
{
  write_IO_value(SETTING_GPO1, buf);
  return len;
}
//-


static ssize_t ControlVersion_read(struct device *dev,struct device_attribute *attr, char *buf)
{
  return sprintf(buf, "%s\n", CONTROL_DRIVER_VERSION);
}

static DEVICE_ATTR(tp8m_GPO_GPO2, 0660, tp8m_GPO_GPO2_read, tp8m_GPO_GPO2_write);
static DEVICE_ATTR(tp8m_GPO_GPO1, 0660, tp8m_GPO_GPO1_read, tp8m_GPO_GPO1_write);
static DEVICE_ATTR(ControlVersion, 	0770, ControlVersion_read, NULL);


static struct attribute *DEVICE_NAME_attributes[] = {
	&dev_attr_tp8m_GPO_GPO2.attr,
	&dev_attr_tp8m_GPO_GPO1.attr,
	&dev_attr_ControlVersion.attr,
	NULL
};


static const struct attribute_group DEVICE_NAME_attr_group = 
{
	.name = "tp8m",
	.attrs = DEVICE_NAME_attributes,
};


static void init(void)
{

}


static int tp8m_init(void)
{
	init();
  pdev = platform_device_register_simple(DEVICE_NAME, -1, NULL, 0);
  return sysfs_create_group(&pdev->dev.kobj, &DEVICE_NAME_attr_group);
}

static void tp8m_cleanup(void)
{
  sysfs_remove_group(&pdev->dev.kobj, &DEVICE_NAME_attr_group);
  platform_device_unregister(pdev);
}



module_init(tp8m_init);
module_exit(tp8m_cleanup);
MODULE_LICENSE("GPL");

