#!/bin/bash
# ===========================================================================
# Copyright (c) 2018,Infogain Corp. All rights reserved.
# ===========================================================================
# $Author: Jai Kumar $
# ===========================================================================
#############################################################################

### COLOR CODES 
red=`tput setaf 1`
green=`tput setaf 2`
reset=`tput sgr0`


INSTALL_DIR=/opt
#JAVA_VERSION_REQUIRED=1\.8
current_time=$(date "+%Y.%m.%d-%H.%M.%S")

FILE_NAME=$0
CURRENT_DIR=$(pwd)
PACKAGE_DIR=${FILE_NAME%install.sh*}
if [ "$1" != "" ] ; then
INSTALL_DIR=$1
fi
log_file=$CURRENT_DIR/log_$current_time.log
echo $JAVA_HOME
#set JAVA_VERSION_REQUIRED
if type -p java; then
    echo "*	Java executable found in PATH   " >> $log_file
    _java=java
elif [[ -n "$JAVA_HOME" ]] && [[ -x "$JAVA_HOME/bin/java" ]];  then
    echo "*	Java executable found in JAVA_HOME   " >> $log_file  
    _java="$JAVA_HOME/bin/java"
else
    echo "*	Install Java to proceed  " >> $log_file
fi

if [[ "$_java" ]]; then
    version=$("$_java" -version 2>&1 | awk -F '"' '/version/ {print $2}')
    echo "*	Java version is $version  " >> $log_file
    if [[ "$version" > "1.5" || "$version" < "1.9" ]]; then
	echo "*	Java version lies between 1.6 or greater   " >> $log_file
	JAVA_VERSION_REQUIRED=$version
    else         
        echo "*	Java version do not lie between 1.6 or greater   " >> $log_file
    fi
fi

tput setaf 2;  echo '*******************************************************************************************' >> $log_file
echo "*		Installation process has been started at $INSTALL_DIR on    " >> $log_file
echo "*		$(date)                             " >> $log_file
echo '*******************************************************************************************' >> $log_file

# Validate Installation Directory exist or not. Create installation directory if not already exists
if [ ! -d "$INSTALL_DIR" ]; then
		echo $INSTALL_DIR does not exist. Creating $INSTALL_DIR >> $log_file
		mkdir $INSTALL_DIR
		chmod 777 $INSTALL_DIR
		a=1
		echo 'Installation in progress' >> $log_file
elif [ -d "$INSTALL_DIR" ]; then
		a=1
		echo 'Installation in progress' >> $log_file
fi

# Validate the Java version exist or not
if [ "$JAVA_HOME" = "" ] ; then
        tput setaf 1;  echo "You must set JAVA_HOME"  >> $log_file 
		tput sgr0;
        exit 1;
    else
    	$JAVA_HOME/bin/java -version 2>&1 | grep $JAVA_VERSION_REQUIRED > /dev/null
    if [ $? -ne 0 ]
    then
    	tput setaf 1;  echo "JAVA_HOME must contain Java $JAVA_VERSION_REQUIRED"  >> $log_file
		tput sgr0;
    	exit 1
    fi
fi


### Install the package
if [ "$(find . -name Fujitsu32.tar.gz)" == "" ] && [ "$(find . -name Fujitsu64.tar.gz)" == "" ]; then
		tput setaf 1;  echo "No Package exist. Please validate source contains Fujitsu32.tar.gz or Fujitsu64.tar.gz"  >> $log_file
		tput sgr0;
        exit 1;
else
		if [ -d "$INSTALL_DIR/Fujitsu" ]; then
			
			

			BKUP_FILE=Fujitsu_$current_time.tgz
			echo "Package already installed. Creating Backup $BKUP_FILE"  >> $log_file
			cd $INSTALL_DIR
			tar --create --gzip --file=$BKUP_FILE Fujitsu
			sudo rm -Rf Fujitsu
			cd $CURRENT_DIR
		fi	
		
		echo $PACKAGE_DIR >> $log_file
		echo $INSTALL_DIR >> $log_file
		echo $CURRENT_DIR >> $log_file

		is64BitJava=$($JAVA_HOME/bin/java -d64 -version 2>&1 | awk '/not support/ {print}')
		
		if [ "$is64BitJava" == "" ]; then
			echo "installation for 64 bit" >> $log_file
			tar -zxvf $PACKAGE_DIR/Fujitsu64.tar.gz -C $INSTALL_DIR >> $log_file
		else
			echo "installation for 32 bit" >> $log_file
			tar -zxvf $PACKAGE_DIR/Fujitsu32.tar.gz -C $INSTALL_DIR >> $log_file
		fi
		
		## Create kernel object and install it.
			if [ -d "/sys/devices/platform/tp8m/tp8m" ]; then
				tput setaf 1;  echo "Driver already installed on this computer" >> $log_file
				sudo /sbin/rmmod tp8m.ko
			fi
				cd $INSTALL_DIR/Fujitsu/JavaPOS/lib
				set -e
				#make -f Makefile 
				sudo make >> $log_file 
				sudo /sbin/insmod tp8m.ko >> $log_file
				sudo chmod 666 /sys/devices/platform/tp8m/tp8m/* >> $log_file
				
				if [ ! -f /etc/init.d/after.local ]; then
					echo "after.local does not exist" >> $log_file
					touch /etc/init.d/after.local
					chmod 666 /etc/init.d/after.local
					echo "after.local created" >> $log_file
				fi
				grep -q -F $INSTALL_DIR'/Fujitsu/JavaPOS/lib' /etc/init.d/after.local || echo 'cd ' $INSTALL_DIR'/Fujitsu/JavaPOS/lib' >> /etc/init.d/after.local
				grep -q -F 'sudo /sbin/insmod tp8m.ko' /etc/init.d/after.local || echo 'sudo /sbin/insmod tp8m.ko' >> /etc/init.d/after.local
				grep -q -F 'sudo chmod 666 /sys/devices/platform/tp8m/tp8m/*' /etc/init.d/after.local || echo 'sudo chmod 666 /sys/devices/platform/tp8m/tp8m/*' >> /etc/init.d/after.local
				cd $CURRENT_DIR >> $log_file
	
		echo '*******************************************************************************************'  >> $log_file
		echo '*  		INSTALLATION SUCCESSFULLY COMPLETED                              '  >> $log_file
		echo '*******************************************************************************************'  >> $log_file
	
fi
tput sgr0;
